import json
from openai import OpenAI

client = OpenAI()
from pydantic import BaseModel

class AutoFunc():
    def __init__(self, system_instruction: str,
                 response_model: BaseModel = None,
                 model: str = "gpt-4-0613",
                 temperature: int = 0
                ):
        self.client = OpenAI()
        self.system_instruction = system_instruction
        self.model = model
        self.temperature = temperature
        self.response_model = response_model
        self.schema = response_model.model_json_schema() if response_model else None

    def __call__(self, input):
        params = {
            'model': self.model,
            'temperature': self.temperature,
            'messages': [
                {"role": "system", "content": self.system_instruction},
                {"role": "user", "content": input},
            ],
        }

        # Use function calling only if a schema is provided
        if self.schema:
            params |= {
                'functions': [
                    {
                      "name": "fn",
                      "description": "GPT function",
                      "parameters": self.schema
                    }
                ],
                'function_call': {"name": "fn"}
            }

        response = self.client.chat.completions.create(**params)
        return response.choices[0].message.content \
            if self.schema is None \
            else json.loads(response.choices[0].message.function_call.arguments)



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) 2023 Nagarjuna Kumarappan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# Author: Nagarjuna Kumarappan <nagarjuna.412@gmail.com>

import os
from setuptools import setup
from distutils.command.install import INSTALL_SCHEMES

for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

prjdir = os.path.dirname(__file__)

def read(filename):
    return open(os.path.join(prjdir, filename)).read()

LONG_DESC = read('README.md') + '\nCHANGES\n=======\n\n' + read('CHANGES')

setup(name='autofunc',
      include_package_data=True,
      version=read('VERSION').strip('\n'),
      description="Easily define python functions for structured LLM output",
      long_description=LONG_DESC,
      long_description_content_type='text/markdown',
      author='Nagarjuna Kumarappan',
      license='MIT',
      author_email='nagarjuna.412@gmail.com',
      url='https://github.com/nk412/autofunc',
      install_requires=[
        'pydantic==2.4.2',
        'openai==1.12.0',
      ],
      py_modules=['autofunc'],
      packages=['autofunc'],
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          ]
      )

